==> spider-tool-1.0.17.tar.gz: spider-tool-1.0.17/spider_tool/crawler.py <==
import requests
# from selenium import webdriver
from urllib.parse import urlencode
from .toolkit import Toolkit
from .log4p import logger
import time
from .metrics import METRICS_RENDER_SUCCESS_200_COUNTER, RENDER_LATENCY, METRICS_RENDER_SUCCESS_COUNTER, \
    METRICS_RENDER_FAIL_COUNTER

from requests.packages.urllib3.exceptions import InsecureRequestWarning

# 禁用安全请求警告
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class JSRenderError(Exception):
    pass


class MethodError(Exception):
    pass


class NoResponseError(Exception):
    pass


# render_url = 'http://yourhost:8050/execute'
def splash_render(render_url, url, headers=None, proxy=None, is_proxy=0, method='GET', parameters_data=None,
                  time_out=5, lua_source=None,
                  wait=1):
    json_body = {
        "headers": {
            #     "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            #     "Accept-Language": "zh-CN,en-US;q=0.8",
            "User-Agent": Toolkit.user_agent(),
        },
        "lua_source": \
            """
                function main(splash, args)
                  splash.resource_timeout = %s
                  assert(splash:go(args.url))
                  assert(splash:wait(1))
                  return splash:html()
                end
            """,
        "http_method": "GET",
        "render_all": 1,
        "timeout": 5,
        "wait": 1
    }
    if lua_source:
        json_body['lua_source'] = lua_source
    else:
        json_body['lua_source'] = json_body['lua_source'] % time_out
    if is_proxy:
        json_body['proxy'] = proxy
    if method.upper() == 'GET' and parameters_data:
        if isinstance(parameters_data, dict):
            url = url + urlencode(parameters_data)
        else:
            url = url + parameters_data
    json_body['url'] = url
    json_body['http_method'] = method.upper()
    json_body['time_out'] = 5
    json_body['wait'] = wait
    if headers:
        json_body['headers'] = headers
    if method.upper() == 'POST' and parameters_data:
        json_body['body'] = parameters_data
    try:
        start_time = time.time()
        html = requests.post(url=render_url, json=json_body, timeout=5)
        end_time = time.time()
        resp_time = end_time - start_time
        RENDER_LATENCY.labels('js').observe(resp_time)
        if html.status_code == 200:
            METRICS_RENDER_SUCCESS_COUNTER.inc()
        else:
            METRICS_RENDER_FAIL_COUNTER.inc()
        return html
    except Exception:
        METRICS_RENDER_FAIL_COUNTER.inc()
        raise JSRenderError


class Response(object):
    def __init__(self):
        self.status_code = 400
        self.encoding = 'utf-8'
        self.text = ''
        self.content = self.text.encode(self.encoding)

    def update(self):
        self.content = self.text.encode(self.encoding)


# def selenium_render(render_url, url, time_out=10):
#     r = Response()
#     try:
#         driver = webdriver.Remote(
#             command_executor='%s/wd/hub' % render_url,
#             desired_capabilities={'browserName': 'chrome'}
#         )
#         driver.set_page_load_timeout(time_out)
#         driver.get(url)
#         r.text = driver.page_source
#         r.status_code = 200
#         driver.close()
#         r.update()
#         METRICS_RENDER_SUCCESS_COUNTER.inc()
#         try:
#             driver.quit()
#         except:
#             pass
#     except Exception as E:
#         try:
#             driver.close()
#             driver.quit()
#         except:
#             pass
#         METRICS_RENDER_FAIL_COUNTER.inc()
#         raise JSRenderError
#     return r


def requests_backend(url, headers, proxies, is_proxy=0, method='GET', parameters_data=None, cookies=None, is_render=0,
                     render_url=None, selenium_url=None, lua_source=None, timeout=10, verify=False):
    if not method:
        method = 'GET'
    method = method.upper()
    if not is_render:
        if method == 'GET':
            if is_proxy:
                response = requests.get(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                        proxies=proxies, params=parameters_data, cookies=cookies)
            else:
                response = requests.get(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                        params=parameters_data, cookies=cookies)
        elif method == 'POST':
            if is_proxy:
                response = requests.post(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                         proxies=proxies, data=parameters_data, cookies=cookies)
            else:
                response = requests.post(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                         data=parameters_data, cookies=cookies)
        else:
            logger.info('请选择get或post作为http请求方式')
            raise MethodError

        if response.status_code == 200:
            response.encoding = response.apparent_encoding
        return response
    else:
        # try:
        #     response = splash_render(render_url=render_url, url=url, headers=None, proxy=proxies.get('http'),
        #                              is_proxy=is_proxy, method=method, parameters_data=parameters_data,
        #                              time_out=timeout,
        #                              lua_source=lua_source, wait=timeout)
        # except JSRenderError:
        #     response = selenium_render(render_url=selenium_url, url=url, time_out=timeout)
        # if response.status_code != 200:
        #     response = selenium_render(render_url=selenium_url, url=url, time_out=timeout)
        response = splash_render(render_url=render_url, url=url, headers=None, proxy=proxies.get('http'),
                                 is_proxy=is_proxy, method=method, parameters_data=parameters_data,
                                 time_out=timeout,
                                 lua_source=lua_source, wait=timeout)
        return response

==> spider-tool-1.0.42.tar.gz: spider-tool-1.0.42/spider_tool/crawler.py <==
import time
import random
import requests
import pychrome
from selenium import webdriver
from urllib.parse import urlencode
from .toolkit import Toolkit
from .log4p import logger
import time
from .metrics import METRICS_RENDER_SUCCESS_200_COUNTER, RENDER_LATENCY, METRICS_RENDER_SUCCESS_COUNTER, \
    METRICS_RENDER_FAIL_COUNTER

from requests.packages.urllib3.exceptions import InsecureRequestWarning

# 禁用安全请求警告
requests.packages.urllib3.disable_warnings(InsecureRequestWarning)


class JSRenderError(Exception):
    pass


class MethodError(Exception):
    pass


class NoResponseError(Exception):
    pass


# render_url = 'http://yourhost:8050/execute'
def splash_render(render_url, url, headers=None, proxy=None, is_proxy=0, method='GET', parameters_data=None,
                  time_out=5, lua_source=None,
                  wait=1):
    json_body = {
        "headers": {
            #     "Accept": "text/html,application/xhtml+xml,application/xml;q=0.9,*/*;q=0.8",
            #     "Accept-Language": "zh-CN,en-US;q=0.8",
            "User-Agent": Toolkit.user_agent(),
        },
        "lua_source": \
            """function main(splash, args)
                  splash.resource_timeout = %s
                  assert(splash:go(args.url))
                  assert(splash:wait(1))
                  return splash:html()
                end
            """,  # lua首行不能有空行
        "http_method": "GET",
        "render_all": 1,
        "timeout": 5,
        "wait": 1
    }
    if lua_source:
        json_body['lua_source'] = lua_source
    else:
        json_body['lua_source'] = json_body['lua_source'] % time_out
    if is_proxy:
        json_body['proxy'] = proxy
    if method.upper() == 'GET' and parameters_data:
        if isinstance(parameters_data, dict):
            url = url + urlencode(parameters_data)
        else:
            url = url + parameters_data
    json_body['url'] = url
    json_body['http_method'] = method.upper()
    json_body['time_out'] = 5
    json_body['wait'] = wait
    if headers:
        json_body['headers'] = headers
    if method.upper() == 'POST' and parameters_data:
        json_body['body'] = parameters_data
    try:
        start_time = time.time()
        html = requests.post(url=render_url, json=json_body, timeout=5, verify=False)
        end_time = time.time()
        resp_time = end_time - start_time
        RENDER_LATENCY.labels('js').observe(resp_time)
        if html.status_code == 200:
            METRICS_RENDER_SUCCESS_COUNTER.inc()
        else:
            METRICS_RENDER_FAIL_COUNTER.inc()
        return html
    except Exception:
        METRICS_RENDER_FAIL_COUNTER.inc()
        raise JSRenderError


class Response(object):
    def __init__(self):
        self.status_code = 400
        self.encoding = 'utf-8'
        self.text = ''
        self.content = self.text.encode(self.encoding)

    def update(self):
        self.content = self.text.encode(self.encoding)


def selenium_render(render_url, url, time_out=10):
    r = Response()
    try:
        driver = webdriver.Remote(
            command_executor='%s/wd/hub' % render_url,
            desired_capabilities={'browserName': 'chrome'}
        )
        driver.set_page_load_timeout(time_out)
        driver.get(url)
        r.text = driver.page_source
        r.status_code = 200
        driver.quit()
        r.update()
        METRICS_RENDER_SUCCESS_COUNTER.inc()
        try:
            driver.quit()
        except:
            pass
    except Exception as E:
        try:
            driver.quit()
        except:
            pass
        METRICS_RENDER_FAIL_COUNTER.inc()
        raise JSRenderError
    return r


def py_chrome_render(render_url, url, time_out=10):
    r = Response()
    browser = pychrome.Browser(url=render_url)
    tab = browser.new_tab()

    def request_will_be_sent(**kwargs):
        print("loading: %s" % kwargs.get('request').get('url'))

    tab.Network.requestWillBeSent = request_will_be_sent
    tab.start()
    tab.Network.enable()
    tab.Page.navigate(url=url, _timeout=time_out)
    html = tab.Runtime.evaluate(expression="document.documentElement.outerHTML")
    if html.get('result'):
        html = html.get('result').get('value')
        r.text = html
        r.status_code = 200
        r.update()
        METRICS_RENDER_SUCCESS_COUNTER.inc()
        tab.stop()
        browser.close_tab(tab)
    else:
        METRICS_RENDER_FAIL_COUNTER.inc()
        tab.stop()
        browser.close_tab(tab)
        raise JSRenderError
    return r


def requests_backend(url, headers, proxies, is_proxy=0, method='GET', parameters_data=None, cookies=None, is_render=0,
                     render_url=None, selenium_url=None, pychrome_url=None, lua_source=None, timeout=10, verify=False):
    if not method:
        method = 'GET'
    method = method.upper()
    if not is_render:
        if method == 'GET':
            if is_proxy:
                response = requests.get(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                        proxies=proxies, params=parameters_data, cookies=cookies)
            else:
                response = requests.get(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                        params=parameters_data, cookies=cookies)
        elif method == 'POST':
            if is_proxy:
                response = requests.post(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                         proxies=proxies, data=parameters_data, cookies=cookies)
            else:
                response = requests.post(url=url, timeout=(timeout, timeout), verify=verify, headers=headers,
                                         data=parameters_data, cookies=cookies)
        else:
            logger.info('请选择get或post作为http请求方式')
            raise MethodError

        if response.status_code == 200:
            response.encoding = response.apparent_encoding
        return response
    else:
        # try:
        #     response = splash_render(render_url=render_url, url=url, headers=None, proxy=proxies.get('http'),
        #                              is_proxy=is_proxy, method=method, parameters_data=parameters_data,
        #                              time_out=timeout,
        #                              lua_source=lua_source, wait=timeout)
        # except JSRenderError:
        #     response = selenium_render(render_url=selenium_url, url=url, time_out=timeout)
        # if response.status_code != 200:
        #     response = selenium_render(render_url=selenium_url, url=url, time_out=timeout)
        if 'cde' in url or 'CDE' in url:
            time.sleep(random.randrange(1, 10))
            response = py_chrome_render(render_url=render_url, url=url, time_out=timeout)
        else:
            response = splash_render(render_url=render_url, url=url, headers=None, proxy=proxies.get('http'),
                                     is_proxy=is_proxy, method=method, parameters_data=parameters_data,
                                     time_out=timeout,
                                     lua_source=lua_source, wait=timeout)
        return response
